Package: keras
Type: Package
Title: R Interface to 'Keras'
Version: 2.16.1
Authors@R: c(
  person("Tomasz", "Kalinowski", role = c("ctb", "cph", "cre"),
         email = "tomasz@posit.co"),
  person("Daniel", "Falbel", role = c("ctb", "cph"), email = "daniel@rstudio.com"),
  person("JJ", "Allaire", role = c("aut", "cph")),
  person("François", "Chollet", role = c("aut", "cph")),
  person("RStudio", role = c("ctb", "cph", "fnd")),
  person("Google", role = c("ctb", "cph", "fnd")),
  person("Yuan", "Tang", role = c("ctb", "cph"),
         email = "terrytangyuan@gmail.com",
         comment = c(ORCID = "0000-0001-5243-233X")),
  person("Wouter", "Van Der Bijl", role = c("ctb", "cph")),
  person("Martin", "Studer", role = c("ctb", "cph")),
  person("Sigrid", "Keydana", role = c("ctb"))
  )
Description: Interface to 'Keras' <https://keras.io>, a high-level neural
  networks 'API'. 'Keras' was developed with a focus on enabling fast experimentation,
  supports both convolution based networks and recurrent networks (as well as
  combinations of the two), and runs seamlessly on both 'CPU' and 'GPU' devices.
Encoding: UTF-8
License: MIT + file LICENSE
URL: https://tensorflow.rstudio.com/,
        https://github.com/rstudio/keras3/tree/r2
BugReports: https://github.com/rstudio/keras3/issues
Depends: R (>= 3.6)
Imports: generics (>= 0.0.1), reticulate (>= 1.31), tensorflow (>=
        2.13.0.9000), tfruns (>= 1.0), magrittr, zeallot, glue,
        methods, R6, rlang
Suggests: ggplot2, testthat (>= 2.1.0), knitr, rmarkdown, callr,
        tfdatasets, withr, png, jpeg
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-02-13 18:30:34 UTC; tomasz
Author: Tomasz Kalinowski [ctb, cph, cre],
  Daniel Falbel [ctb, cph],
  JJ Allaire [aut, cph],
  François Chollet [aut, cph],
  RStudio [ctb, cph, fnd],
  Google [ctb, cph, fnd],
  Yuan Tang [ctb, cph] (ORCID: <https://orcid.org/0000-0001-5243-233X>),
  Wouter Van Der Bijl [ctb, cph],
  Martin Studer [ctb, cph],
  Sigrid Keydana [ctb]
Maintainer: Tomasz Kalinowski <tomasz@posit.co>
Repository: CRAN
Date/Publication: 2026-02-14 06:10:03 UTC
Built: R 4.6.0; ; 2026-04-28 23:19:56 UTC; unix
