# Generated by roxygen2: do not edit by hand

S3method("$",py_R6ClassGenerator)
S3method(as.data.frame,keras_training_history)
S3method(as_generator,"function")
S3method(as_generator,default)
S3method(as_generator,keras_preprocessing.sequence.TimeseriesGenerator)
S3method(as_generator,tensorflow.python.data.ops.dataset_ops.Dataset)
S3method(as_generator,tensorflow.python.data.ops.dataset_ops.DatasetV2)
S3method(compile,keras.engine.training.Model)
S3method(compose_layer,default)
S3method(compose_layer,keras.engine.sequential.Sequential)
S3method(compose_layer,keras.models.Sequential)
S3method(evaluate,keras.engine.training.Model)
S3method(export_savedmodel,keras.engine.training.Model)
S3method(fit,keras.engine.training.Model)
S3method(format,keras.engine.training.Model)
S3method(is_main_thread_generator,"function")
S3method(is_main_thread_generator,default)
S3method(is_main_thread_generator,keras.preprocessing.image.Iterator)
S3method(is_main_thread_generator,keras_preprocessing.image.Iterator)
S3method(is_main_thread_generator,keras_preprocessing.image.iterator.Iterator)
S3method(is_main_thread_generator,keras_preprocessing.sequence.TimeseriesGenerator)
S3method(is_main_thread_generator,tensorflow.python.data.ops.dataset_ops.Dataset)
S3method(pillar::type_sum,py_R6ClassGenerator)
S3method(plot,keras.engine.training.Model)
S3method(plot,keras_training_history)
S3method(predict,keras.engine.training.Model)
S3method(print,keras.engine.training.Model)
S3method(print,keras_training_history)
S3method(print,kerastools.model.RModel)
S3method(print,py_R6ClassGenerator)
S3method(py_str,keras.engine.training.Model)
S3method(py_to_r,keras.utils.generic_utils.SharedObjectConfig)
S3method(py_to_r_wrapper,keras.engine.base_layer.Layer)
S3method(py_to_r_wrapper,keras.engine.training.Model)
S3method(py_to_r_wrapper,kerastools.model.RModel)
S3method(r_to_py,R6ClassGenerator)
S3method(r_to_py,keras_layer_wrapper)
S3method(summary,keras.engine.training.Model)
S3method(summary,kerastools.model.RModel)
export("%<-%")
export("%<-active%")
export("%<>%")
export("%>%")
export("%py_class%")
export(KerasCallback)
export(KerasConstraint)
export(KerasLayer)
export(KerasWrapper)
export(Layer)
export(activation_elu)
export(activation_exponential)
export(activation_gelu)
export(activation_hard_sigmoid)
export(activation_linear)
export(activation_relu)
export(activation_selu)
export(activation_sigmoid)
export(activation_softmax)
export(activation_softplus)
export(activation_softsign)
export(activation_swish)
export(activation_tanh)
export(adapt)
export(application_densenet)
export(application_densenet121)
export(application_densenet169)
export(application_densenet201)
export(application_efficientnet_b0)
export(application_efficientnet_b1)
export(application_efficientnet_b2)
export(application_efficientnet_b3)
export(application_efficientnet_b4)
export(application_efficientnet_b5)
export(application_efficientnet_b6)
export(application_efficientnet_b7)
export(application_inception_resnet_v2)
export(application_inception_v3)
export(application_mobilenet)
export(application_mobilenet_v2)
export(application_mobilenet_v3_large)
export(application_mobilenet_v3_small)
export(application_nasnet)
export(application_nasnetlarge)
export(application_nasnetmobile)
export(application_resnet101)
export(application_resnet101_v2)
export(application_resnet152)
export(application_resnet152_v2)
export(application_resnet50)
export(application_resnet50_v2)
export(application_vgg16)
export(application_vgg19)
export(application_xception)
export(array_reshape)
export(as_tensor)
export(backend)
export(bidirectional)
export(callback_backup_and_restore)
export(callback_csv_logger)
export(callback_early_stopping)
export(callback_lambda)
export(callback_learning_rate_scheduler)
export(callback_model_checkpoint)
export(callback_progbar_logger)
export(callback_reduce_lr_on_plateau)
export(callback_remote_monitor)
export(callback_tensorboard)
export(callback_terminate_on_naan)
export(clone_model)
export(compile)
export(constraint_maxnorm)
export(constraint_minmaxnorm)
export(constraint_nonneg)
export(constraint_unitnorm)
export(count_params)
export(create_layer)
export(create_layer_wrapper)
export(create_wrapper)
export(custom_metric)
export(dataset_boston_housing)
export(dataset_cifar10)
export(dataset_cifar100)
export(dataset_fashion_mnist)
export(dataset_imdb)
export(dataset_imdb_word_index)
export(dataset_mnist)
export(dataset_reuters)
export(dataset_reuters_word_index)
export(densenet_preprocess_input)
export(evaluate)
export(evaluate_generator)
export(export_savedmodel)
export(fit)
export(fit_generator)
export(fit_image_data_generator)
export(fit_text_tokenizer)
export(flag_boolean)
export(flag_integer)
export(flag_numeric)
export(flag_string)
export(flags)
export(flow_images_from_data)
export(flow_images_from_dataframe)
export(flow_images_from_directory)
export(freeze_weights)
export(from_config)
export(generator_next)
export(get_config)
export(get_file)
export(get_input_at)
export(get_input_mask_at)
export(get_input_shape_at)
export(get_layer)
export(get_output_at)
export(get_output_mask_at)
export(get_output_shape_at)
export(get_vocabulary)
export(get_weights)
export(hdf5_matrix)
export(image_array_resize)
export(image_array_save)
export(image_data_generator)
export(image_dataset_from_directory)
export(image_load)
export(image_to_array)
export(imagenet_decode_predictions)
export(imagenet_preprocess_input)
export(implementation)
export(inception_resnet_v2_preprocess_input)
export(inception_v3_preprocess_input)
export(initializer_constant)
export(initializer_glorot_normal)
export(initializer_glorot_uniform)
export(initializer_he_normal)
export(initializer_he_uniform)
export(initializer_identity)
export(initializer_lecun_normal)
export(initializer_lecun_uniform)
export(initializer_ones)
export(initializer_orthogonal)
export(initializer_random_normal)
export(initializer_random_uniform)
export(initializer_truncated_normal)
export(initializer_variance_scaling)
export(initializer_zeros)
export(install_keras)
export(is_keras_available)
export(k_abs)
export(k_all)
export(k_any)
export(k_arange)
export(k_argmax)
export(k_argmin)
export(k_backend)
export(k_batch_dot)
export(k_batch_flatten)
export(k_batch_get_value)
export(k_batch_normalization)
export(k_batch_set_value)
export(k_bias_add)
export(k_binary_crossentropy)
export(k_cast)
export(k_cast_to_floatx)
export(k_categorical_crossentropy)
export(k_clear_session)
export(k_clip)
export(k_concatenate)
export(k_constant)
export(k_conv1d)
export(k_conv2d)
export(k_conv2d_transpose)
export(k_conv3d)
export(k_conv3d_transpose)
export(k_cos)
export(k_count_params)
export(k_ctc_batch_cost)
export(k_ctc_decode)
export(k_ctc_label_dense_to_sparse)
export(k_cumprod)
export(k_cumsum)
export(k_depthwise_conv2d)
export(k_dot)
export(k_dropout)
export(k_dtype)
export(k_elu)
export(k_epsilon)
export(k_equal)
export(k_eval)
export(k_exp)
export(k_expand_dims)
export(k_eye)
export(k_flatten)
export(k_floatx)
export(k_foldl)
export(k_foldr)
export(k_function)
export(k_gather)
export(k_get_session)
export(k_get_uid)
export(k_get_value)
export(k_get_variable_shape)
export(k_gradients)
export(k_greater)
export(k_greater_equal)
export(k_hard_sigmoid)
export(k_identity)
export(k_image_data_format)
export(k_in_test_phase)
export(k_in_top_k)
export(k_in_train_phase)
export(k_int_shape)
export(k_is_keras_tensor)
export(k_is_placeholder)
export(k_is_sparse)
export(k_is_tensor)
export(k_l2_normalize)
export(k_learning_phase)
export(k_less)
export(k_less_equal)
export(k_local_conv1d)
export(k_local_conv2d)
export(k_log)
export(k_logsumexp)
export(k_manual_variable_initialization)
export(k_map_fn)
export(k_max)
export(k_maximum)
export(k_mean)
export(k_min)
export(k_minimum)
export(k_moving_average_update)
export(k_ndim)
export(k_normalize_batch_in_training)
export(k_not_equal)
export(k_one_hot)
export(k_ones)
export(k_ones_like)
export(k_permute_dimensions)
export(k_placeholder)
export(k_pool2d)
export(k_pool3d)
export(k_pow)
export(k_print_tensor)
export(k_prod)
export(k_random_bernoulli)
export(k_random_binomial)
export(k_random_normal)
export(k_random_normal_variable)
export(k_random_uniform)
export(k_random_uniform_variable)
export(k_relu)
export(k_repeat)
export(k_repeat_elements)
export(k_reset_uids)
export(k_reshape)
export(k_resize_images)
export(k_resize_volumes)
export(k_reverse)
export(k_rnn)
export(k_round)
export(k_separable_conv2d)
export(k_set_epsilon)
export(k_set_floatx)
export(k_set_image_data_format)
export(k_set_learning_phase)
export(k_set_session)
export(k_set_value)
export(k_shape)
export(k_sigmoid)
export(k_sign)
export(k_sin)
export(k_softmax)
export(k_softplus)
export(k_softsign)
export(k_sparse_categorical_crossentropy)
export(k_spatial_2d_padding)
export(k_spatial_3d_padding)
export(k_sqrt)
export(k_square)
export(k_squeeze)
export(k_stack)
export(k_std)
export(k_stop_gradient)
export(k_sum)
export(k_switch)
export(k_tanh)
export(k_temporal_padding)
export(k_tile)
export(k_to_dense)
export(k_transpose)
export(k_truncated_normal)
export(k_unstack)
export(k_update)
export(k_update_add)
export(k_update_sub)
export(k_var)
export(k_variable)
export(k_zeros)
export(k_zeros_like)
export(keras)
export(keras_array)
export(keras_model)
export(keras_model_custom)
export(keras_model_sequential)
export(layer_activation)
export(layer_activation_elu)
export(layer_activation_leaky_relu)
export(layer_activation_parametric_relu)
export(layer_activation_relu)
export(layer_activation_selu)
export(layer_activation_softmax)
export(layer_activation_thresholded_relu)
export(layer_activity_regularization)
export(layer_add)
export(layer_additive_attention)
export(layer_alpha_dropout)
export(layer_attention)
export(layer_average)
export(layer_average_pooling_1d)
export(layer_average_pooling_2d)
export(layer_average_pooling_3d)
export(layer_batch_normalization)
export(layer_category_encoding)
export(layer_center_crop)
export(layer_concatenate)
export(layer_conv_1d)
export(layer_conv_1d_transpose)
export(layer_conv_2d)
export(layer_conv_2d_transpose)
export(layer_conv_3d)
export(layer_conv_3d_transpose)
export(layer_conv_lstm_1d)
export(layer_conv_lstm_2d)
export(layer_conv_lstm_3d)
export(layer_cropping_1d)
export(layer_cropping_2d)
export(layer_cropping_3d)
export(layer_cudnn_gru)
export(layer_cudnn_lstm)
export(layer_dense)
export(layer_dense_features)
export(layer_depthwise_conv_1d)
export(layer_depthwise_conv_2d)
export(layer_discretization)
export(layer_dot)
export(layer_dropout)
export(layer_embedding)
export(layer_flatten)
export(layer_gaussian_dropout)
export(layer_gaussian_noise)
export(layer_global_average_pooling_1d)
export(layer_global_average_pooling_2d)
export(layer_global_average_pooling_3d)
export(layer_global_max_pooling_1d)
export(layer_global_max_pooling_2d)
export(layer_global_max_pooling_3d)
export(layer_gru)
export(layer_gru_cell)
export(layer_hashing)
export(layer_input)
export(layer_integer_lookup)
export(layer_lambda)
export(layer_layer_normalization)
export(layer_locally_connected_1d)
export(layer_locally_connected_2d)
export(layer_lstm)
export(layer_lstm_cell)
export(layer_masking)
export(layer_max_pooling_1d)
export(layer_max_pooling_2d)
export(layer_max_pooling_3d)
export(layer_maximum)
export(layer_minimum)
export(layer_multi_head_attention)
export(layer_multiply)
export(layer_normalization)
export(layer_permute)
export(layer_random_brightness)
export(layer_random_contrast)
export(layer_random_crop)
export(layer_random_flip)
export(layer_random_height)
export(layer_random_rotation)
export(layer_random_translation)
export(layer_random_width)
export(layer_random_zoom)
export(layer_repeat_vector)
export(layer_rescaling)
export(layer_reshape)
export(layer_resizing)
export(layer_rnn)
export(layer_separable_conv_1d)
export(layer_separable_conv_2d)
export(layer_simple_rnn)
export(layer_simple_rnn_cell)
export(layer_spatial_dropout_1d)
export(layer_spatial_dropout_2d)
export(layer_spatial_dropout_3d)
export(layer_stacked_rnn_cells)
export(layer_string_lookup)
export(layer_subtract)
export(layer_text_vectorization)
export(layer_unit_normalization)
export(layer_upsampling_1d)
export(layer_upsampling_2d)
export(layer_upsampling_3d)
export(layer_zero_padding_1d)
export(layer_zero_padding_2d)
export(layer_zero_padding_3d)
export(learning_rate_schedule_cosine_decay)
export(learning_rate_schedule_cosine_decay_restarts)
export(learning_rate_schedule_exponential_decay)
export(learning_rate_schedule_inverse_time_decay)
export(learning_rate_schedule_piecewise_constant_decay)
export(learning_rate_schedule_polynomial_decay)
export(load_model_hdf5)
export(load_model_tf)
export(load_model_weights_hdf5)
export(load_model_weights_tf)
export(load_text_tokenizer)
export(loss_binary_crossentropy)
export(loss_categorical_crossentropy)
export(loss_categorical_hinge)
export(loss_cosine_proximity)
export(loss_cosine_similarity)
export(loss_hinge)
export(loss_huber)
export(loss_kl_divergence)
export(loss_kullback_leibler_divergence)
export(loss_logcosh)
export(loss_mean_absolute_error)
export(loss_mean_absolute_percentage_error)
export(loss_mean_squared_error)
export(loss_mean_squared_logarithmic_error)
export(loss_poisson)
export(loss_sparse_categorical_crossentropy)
export(loss_squared_hinge)
export(make_sampling_table)
export(mark_active)
export(metric_accuracy)
export(metric_auc)
export(metric_binary_accuracy)
export(metric_binary_crossentropy)
export(metric_categorical_accuracy)
export(metric_categorical_crossentropy)
export(metric_categorical_hinge)
export(metric_cosine_proximity)
export(metric_cosine_similarity)
export(metric_false_negatives)
export(metric_false_positives)
export(metric_hinge)
export(metric_kullback_leibler_divergence)
export(metric_logcosh_error)
export(metric_mean)
export(metric_mean_absolute_error)
export(metric_mean_absolute_percentage_error)
export(metric_mean_iou)
export(metric_mean_relative_error)
export(metric_mean_squared_error)
export(metric_mean_squared_logarithmic_error)
export(metric_mean_tensor)
export(metric_mean_wrapper)
export(metric_poisson)
export(metric_precision)
export(metric_precision_at_recall)
export(metric_recall)
export(metric_recall_at_precision)
export(metric_root_mean_squared_error)
export(metric_sensitivity_at_specificity)
export(metric_sparse_categorical_accuracy)
export(metric_sparse_categorical_crossentropy)
export(metric_sparse_top_k_categorical_accuracy)
export(metric_specificity_at_sensitivity)
export(metric_squared_hinge)
export(metric_sum)
export(metric_top_k_categorical_accuracy)
export(metric_true_negatives)
export(metric_true_positives)
export(mobilenet_decode_predictions)
export(mobilenet_load_model_hdf5)
export(mobilenet_preprocess_input)
export(mobilenet_v2_decode_predictions)
export(mobilenet_v2_load_model_hdf5)
export(mobilenet_v2_preprocess_input)
export(model_from_json)
export(model_from_saved_model)
export(model_from_yaml)
export(model_to_json)
export(model_to_saved_model)
export(model_to_yaml)
export(multi_gpu_model)
export(nasnet_preprocess_input)
export(new_callback_class)
export(new_layer_class)
export(new_learning_rate_schedule_class)
export(new_loss_class)
export(new_metric_class)
export(new_model_class)
export(normalize)
export(optimizer_adadelta)
export(optimizer_adagrad)
export(optimizer_adam)
export(optimizer_adamax)
export(optimizer_ftrl)
export(optimizer_nadam)
export(optimizer_rmsprop)
export(optimizer_sgd)
export(pad_sequences)
export(pop_layer)
export(predict_classes)
export(predict_generator)
export(predict_on_batch)
export(predict_proba)
export(py_require_legacy_keras)
export(regularizer_l1)
export(regularizer_l1_l2)
export(regularizer_l2)
export(regularizer_orthogonal)
export(reset_states)
export(resnet_preprocess_input)
export(resnet_v2_preprocess_input)
export(run_dir)
export(save_model_hdf5)
export(save_model_tf)
export(save_model_weights_hdf5)
export(save_model_weights_tf)
export(save_text_tokenizer)
export(sequences_to_matrix)
export(serialize_model)
export(set_vocabulary)
export(set_weights)
export(shape)
export(skipgrams)
export(tensorboard)
export(test_on_batch)
export(text_dataset_from_directory)
export(text_hashing_trick)
export(text_one_hot)
export(text_to_word_sequence)
export(text_tokenizer)
export(texts_to_matrix)
export(texts_to_sequences)
export(texts_to_sequences_generator)
export(time_distributed)
export(timeseries_dataset_from_array)
export(timeseries_generator)
export(to_categorical)
export(train_on_batch)
export(tuple)
export(unfreeze_weights)
export(unserialize_model)
export(use_backend)
export(use_condaenv)
export(use_implementation)
export(use_python)
export(use_session_with_seed)
export(use_virtualenv)
export(with_custom_object_scope)
export(xception_preprocess_input)
export(zip_lists)
import(R6)
import(magrittr)
import(methods)
import(zeallot)
importFrom(generics,compile)
importFrom(generics,fit)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(magrittr,"%<>%")
importFrom(reticulate,array_reshape)
importFrom(reticulate,dict)
importFrom(reticulate,import)
importFrom(reticulate,import_builtins)
importFrom(reticulate,import_from_path)
importFrom(reticulate,iter_next)
importFrom(reticulate,iterate)
importFrom(reticulate,py_call)
importFrom(reticulate,py_capture_output)
importFrom(reticulate,py_clear_last_error)
importFrom(reticulate,py_dict)
importFrom(reticulate,py_eval)
importFrom(reticulate,py_func)
importFrom(reticulate,py_get_attr)
importFrom(reticulate,py_has_attr)
importFrom(reticulate,py_id)
importFrom(reticulate,py_is_null_xptr)
importFrom(reticulate,py_iterator)
importFrom(reticulate,py_load_object)
importFrom(reticulate,py_save_object)
importFrom(reticulate,py_str)
importFrom(reticulate,py_to_r)
importFrom(reticulate,py_to_r_wrapper)
importFrom(reticulate,r_to_py)
importFrom(reticulate,tuple)
importFrom(reticulate,use_condaenv)
importFrom(reticulate,use_python)
importFrom(reticulate,use_virtualenv)
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(rlang,names2)
importFrom(stats,predict)
importFrom(tensorflow,as_tensor)
importFrom(tensorflow,evaluate)
importFrom(tensorflow,export_savedmodel)
importFrom(tensorflow,install_tensorflow)
importFrom(tensorflow,shape)
importFrom(tensorflow,tensorboard)
importFrom(tensorflow,tf_config)
importFrom(tensorflow,tf_version)
importFrom(tensorflow,use_session_with_seed)
importFrom(tfruns,flag_boolean)
importFrom(tfruns,flag_integer)
importFrom(tfruns,flag_numeric)
importFrom(tfruns,flag_string)
importFrom(tfruns,flags)
importFrom(tfruns,run_dir)
