

set -ex



pip check
export KERAS_BACKEND=torch
python -c "import keras;"
python -c "import keras.backend;"
python -c "import keras.datasets;"
python -c "import keras.layers;"
python -c "import keras.utils;"
export KERAS_BACKEND=
python -c "import os; os.environ['KERAS_BACKEND'] = 'numpy'; import keras;"
python -c "import os; os.environ['KERAS_BACKEND'] = 'jax'; import keras;"
python -c "import os; os.environ['KERAS_BACKEND'] = 'torch'; import keras;"
exit 0
