



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
set KERAS_BACKEND=torch
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -c "import keras;"
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -c "import keras.backend;"
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -c "import keras.datasets;"
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -c "import keras.layers;"
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -c "import keras.utils;"
IF %ERRORLEVEL% NEQ 0 exit /B 1
set KERAS_BACKEND=
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -c "import os; os.environ['KERAS_BACKEND'] = 'torch'; import keras;"
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
